@@ -1,8 +1,8 @@
-## haplotype.R (2014-07-27)
+## haplotype.R (2015-03-05)
 
 ##   Haplotype Extraction, Frequencies, and Networks
 
-## Copyright 2009-2014 Emmanuel Paradis, 2013 Klaus Schliep
+## Copyright 2009-2015 Emmanuel Paradis, 2013 Klaus Schliep
 
 ## This file is part of the R-package `pegas'.
 ## See the file ../DESCRIPTION for licensing issues.
@@ -77,34 +77,17 @@ haplotype.DNAbin <- function(x, labels = NULL, ...)
 {
     nms.x <- deparse(substitute(x))
     if (is.list(x)) x <- as.matrix(x)
-    rownames(x) <- NULL
-    y <- apply(x, 1, rawToChar)
-    n <- length(y)
-    keep <- nhaplo <- 1L
-    no <- list(1L)
-    for (i in 2:n) {
-        already.seen <- FALSE
-        j <- 1L
-        while (j <= nhaplo) {
-            if (y[i] == y[keep[j]]) {
-                no[[j]] <- c(no[[j]], i)
-                already.seen <- TRUE
-                break
-            }
-            j <- j + 1L
-        }
-        if (!already.seen) {
-            keep <- c(keep, i)
-            nhaplo <- nhaplo + 1L
-            no[[nhaplo]] <- i
-        }
-    }
-    obj <- x[keep, ]
+    n <- nrow(x)
+    s <- ncol(x)
+    h <- .C(haplotype_DNAbin, x, n, s, integer(n), NAOK = TRUE)[[4]]
+    u <- h == 0
+    h[u] <- i <- which(u)
+    obj <- x[i, ]
     if (is.null(labels))
-        labels <- as.character(as.roman(1:length(keep)))
+        labels <- as.character(as.roman(seq_along(i)))
     rownames(obj) <- labels
     class(obj) <- c("haplotype", "DNAbin")
-    attr(obj, "index") <- no
+    attr(obj, "index") <- lapply(i, function(x) which(h == x))
     attr(obj, "from") <- nms.x
     obj
 }
@@ -131,6 +114,15 @@ haploFreq <- function(x, fac, split = "_", what = 2, haplo = NULL)
     res
 }
 
+diffHaplo <- function(h, a = 1, b = 2)
+{
+    s <- seg.sites(h[c(a, b), ])
+    x <- h[c(a, b), s]
+    cat("\n", dist.dna(x, "TS"), "transitions,",
+        dist.dna(x, "TV"), "transversions\n\n")
+    data.frame(pos = s, toupper(t(as.character(x))))
+}
+
 haploNet <- function(h, d = NULL)
 {
     if (!inherits(h, "haplotype"))
@@ -138,26 +130,34 @@ haploNet <- function(h, d = NULL)
     freq <- sapply(attr(h, "index"), length)
     n <- length(freq) # number of haplotypes
     link <- matrix(0, 0, 3)
-    if (is.null(d)) d <- dist.dna(h, "N")
+    if (is.null(d)) d <- dist.dna(h, "N", pairwise.deletion = TRUE)
     d <- as.matrix(d)
     d[col(d) >= row(d)] <- NA # put NA's in the diag and above-diag elts
-    dimnames(d) <- list(1:n, 1:n)
+    one2n <- seq_len(n)
+    dimnames(d) <- list(one2n, one2n)
     step <- 1
-    gr <- 1:n
+    gr <- one2n
+    altlink <- matrix(0, 0, 3) # alternative links
     while (length(unique(gr)) > 1) {
         newLinks <- which(d == step, TRUE)
         if (length(newLinks)) {
             del <- NULL
-            for (i in 1:nrow(newLinks)) {
-                a <- gr[newLinks[i, 1]]
-                b <- gr[newLinks[i, 2]]
-                ## if both nodes are already in the
-                ## same subnet, then drop this link
-                if (a == b) del <- c(del, i)
-                else gr[which(gr == b)] <- a
+            for (i in seq_len(nrow(newLinks))) {
+                h1 <- newLinks[i, 1]
+                h2 <- newLinks[i, 2]
+                a <- gr[h1]
+                b <- gr[h2]
+                ## if both nodes are already in the same
+                ## subnet, then count the link as alternative
+                if (a == b) {
+                    del <- c(del, i)
+                    if (d[h1, h2] <= step)
+                        altlink <- rbind(altlink, c(newLinks[i, ], step))
+                } else gr[which(gr == b)] <- a
             }
-            if (!is.null(del)) newLinks <- newLinks[-del, , drop=FALSE]
-            link <- rbind(link, cbind(newLinks, rep(step, nrow(newLinks))))
+            if (!is.null(del)) newLinks <- newLinks[-del, , drop = FALSE]
+            newLinks <- cbind(newLinks, rep(step, nrow(newLinks)))
+            link <- rbind(link, newLinks)
         }
         step <- step + 1
     }
@@ -165,15 +165,140 @@ haploNet <- function(h, d = NULL)
     dimnames(link) <- list(NULL, c("", "", "step", "Prob"))
     attr(link, "freq") <- freq
     attr(link, "labels") <- rownames(h)
+    attr(link, "alter.links") <-
+        altlink <- cbind(altlink, .TempletonProb(altlink[, 3], ncol(h)))
     class(link) <- "haploNet"
     link
 }
 
+.drawSymbolsHaploNet <- function(xx, yy, size, col, bg, pie)
+{
+    if (is.null(pie))
+        symbols(xx, yy, circles = size/2, inches = FALSE,
+                add = TRUE, fg = col, bg = bg)
+    else {
+        nc <- ncol(pie)
+        co <- if (length(bg) == 1 && bg == "white") rainbow(nc) else rep(bg, length.out = nc)
+        for (i in seq_along(xx))
+            floating.pie.asp(xx[i], yy[i], pie[i, ], radius = size[i]/2, col = co)
+    }
+}
+
+.drawAlternativeLinks <- function(xx, yy, altlink, threshold, show.mutation)
+{
+    s <- altlink[, 3] >= threshold[1] & altlink[, 3] <= threshold[2]
+    if (!any(s)) return(NULL)
+    xa0 <- xx[altlink[s, 1]]
+    ya0 <- yy[altlink[s, 1]]
+    xa1 <- xx[altlink[s, 2]]
+    ya1 <- yy[altlink[s, 2]]
+    segments(xa0, ya0, xa1, ya1, col = "grey", lty = 2)
+    if (show.mutation)
+        .labelSegmentsHaploNet(xx, yy, altlink[s, 1:2, drop = FALSE],
+                               altlink[s, 3, drop = FALSE], NULL, 1, NULL, 2)
+}
+
+.labelSegmentsHaploNet <- function(xx, yy, link, step, size, lwd, col.link, method)
+{
+### method: the way the segments are labelled
+###   1: Klaus's method
+###   2: show the number of mutations on each link
+
+    l1 <- link[, 1]
+    l2 <- link[, 2]
+    switch(method, {
+        ld1 <- step
+        ld2 <- step# * scale.ratio
+        for (i in seq_along(ld1)) {
+            pc <- ((1:ld1[i]) / (ld1[i] + 1) * ld2[i] + size[l1[i]]/2) / (ld2[i] + (size[l1[i]] + size[l2[i]])/2)
+            xr <- pc * (xx[l2[i]] - xx[l1[i]]) +  xx[l1[i]]
+            yr <- pc * (yy[l2[i]] - yy[l1[i]]) +  yy[l1[i]]
+            symbols(xr, yr, circles = rep(lwd/15, length(xr)), inches = FALSE, add = TRUE,
+                    fg = col.link, bg = col.link)
+        }
+    }, {
+        x <- (xx[l1] + xx[l2])/2
+        y <- (yy[l1] + yy[l2])/2
+        BOTHlabels(step, NULL, x, y, c(0.5, 0.5), "rect", NULL, NULL,
+                   NULL, NULL, "black", "lightgrey", FALSE, NULL, NULL)
+    })
+}
+
+replot <- function(xy = NULL, ...)
+{
+    on.exit(return(list(x = xx, y = yy)))
+
+    Last.phn <- get("last_plot.haploNet", envir = .PlotHaploNetEnv)
+    xx <- Last.phn$xx
+    yy <- Last.phn$yy
+    l1 <- Last.phn$link[, 1]
+    l2 <- Last.phn$link[, 2]
+    step <- Last.phn$step
+    lwd <- Last.phn$lwd
+    size <- Last.phn$size
+    col <- Last.phn$col
+    bg <- Last.phn$bg
+    lty <- Last.phn$lty
+    col.link <- Last.phn$col.link
+    labels <- Last.phn$labels
+    asp <- Last.phn$asp
+    pie <- Last.phn$pie
+    labels <- Last.phn$labels
+    show.mutation <- Last.phn$show.mutation
+    altlink <- Last.phn$alter.links
+    threshold <- Last.phn$threshold
+
+    if (is.character(labels)) {
+        font <- Last.phn$font
+        cex <- Last.phn$cex
+    }
+
+    getMove <- function() {
+        p1 <- locator(1)
+        if (is.null(p1)) return("done")
+        p2 <- locator(1)
+        ## find the closest node to p1...
+        i <- which.min(sqrt((p1$x - xx)^2 + (p1$y - yy)^2))
+        ## ... and change its coordinates:
+        xx[i] <<- p2$x
+        yy[i] <<- p2$y
+    }
+
+    if (is.null(xy)) {
+        cat("Click close to the node you want to move, then click where to place it\n(only single left-clicks).\nRight-click to exit.\n")
+        res <- getMove()
+    } else {
+        xx <- xy$x
+        yy <- xy$y
+        res <- "done"
+    }
+
+    repeat {
+        plot(xx, yy, type = "n", xlab = "", ylab = "",
+             axes = FALSE, bty = "n", asp = asp, ...)
+        segments(xx[l1], yy[l1], xx[l2], yy[l2], lwd = lwd,
+                 lty = lty, col = col.link)
+        if (show.mutation)
+            .labelSegmentsHaploNet(xx, yy, cbind(l1, l2), step, size, lwd,
+                                   col.link, as.numeric(show.mutation))
+        if (!is.null(altlink) && !identical(threshold, 0))
+            .drawAlternativeLinks(xx, yy, altlink, threshold, show.mutation)
+        .drawSymbolsHaploNet(xx, yy, size, col, bg, pie)
+        if (is.character(labels))
+            text(xx, yy, labels, font = font, cex = cex)
+        assign("tmp", list(xx, yy), envir = .PlotHaploNetEnv)
+        eval(quote(last_plot.haploNet[1:2] <- tmp), envir = .PlotHaploNetEnv)
+        if (identical(res, "done")) break
+        res <- getMove()
+    }
+}
+
 plot.haploNet <-
     function(x, size = 1, col = "black", bg = "white",
              col.link = "black", lwd = 1, lty = 1, pie = NULL,
              labels = TRUE, font = 2, cex = 1, scale.ratio = 1,
-             asp = 1, legend = FALSE, fast = FALSE, show.mutation = TRUE, ...)
+             asp = 1, legend = FALSE, fast = FALSE, show.mutation = TRUE,
+             threshold = c(1, 2), ...)
 {
     par(xpd = TRUE)
     link <- x[, 1:2]
@@ -192,8 +317,9 @@ plot.haploNet <-
 
     H <- vector("list", n) # the list of hierarchy of nodes...
 
+    ## the recursive function to allocate quadrants
     foo <- function(i) {
-        j <- NULL # indices of the haplotypes linked to 'i'
+        j <- integer() # indices of the haplotypes linked to 'i'
         for (ii in 1:2) { # look at both columns
             ll <- which(link[, ii] == i & avlb)
             if (length(ll)) {
@@ -203,13 +329,12 @@ plot.haploNet <-
                 avlb[ll] <<- FALSE
             }
         }
-        nlink <- length(j)
-        if (nlink) {
+        if (nlink <- length(j)) {
             H[[i]] <<- j
             start <- theta[i] - angle[i]/2
-            theta[j] <<-
-                seq(start, by = angle[i]/nlink, length.out = nlink)
-            angle[j] <<- angle[i]/nlink # *sqrt(xx[i]^2 + yy[i]^2)
+            by <- angle[i]/nlink
+            theta[j] <<- seq(start, by = by, length.out = nlink)
+            angle[j] <<- by
             xx[j] <<- r[j] * cos(theta[j]) + xx[i]
             yy[j] <<- r[j] * sin(theta[j]) + yy[i]
             for (ii in j) foo(ii)
@@ -221,205 +346,160 @@ plot.haploNet <-
     angle[central] <- 2*pi
     foo(central)
 
-if (!fast) {
-    fCollect <- function(i) {
-        ## find all nodes to move simultaneously
-        ii <- H[[i]]
-        if (!is.null(ii)) {
-            j <<- c(j, ii)
-            for (jj in ii) fCollect(jj)
-        }
-    }
-
-    ## Version qui ne prend pas en compte les angles
-    ## dans le calcul de l'energie du reseau: cela semble
-    ## mieux marcher mais il y a encore des line-crossings
-
-    energy <- function(xx, yy) {
-        ## check line crossings
-        nlink <- length(l1)
-        ## round everyhting to avoid problems:
-        x0 <- round(xx[l1], 1e-6)
-        y0 <- round(yy[l1], 1e-6)
-        x1 <- round(xx[l2], 1e-6)
-        y1 <- round(yy[l2], 1e-6)
-        ## compute all the slopes and intercepts:
-        beta <- (y1 - y0)/(x1 - x0)
-#        browser()
-        intp <- y0 - beta*x0
-        for (i in 1:(nlink - 1)) {
-            for (j in (i + 1):nlink) {
-                ## in case they are nearly parallel:
-                if (any(is.na(beta))) return(Inf)
-                if (beta[i] == beta[j]) next
-                ## if both lines are vertical:
-                if (abs(beta[i]) == Inf && abs(beta[j]) == Inf) next
-                ## find the point where both lines cross
-                ## in case the 1st line is vertical...
-                if (abs(beta[i]) == Inf) {
-                    xi <- x0[i]
-                    yi <- beta[j]*xi + intp[j]
-                } else if (abs(beta[j]) == Inf) { # ... or the 2nd one
-                    xi <- x0[j]
-                    yi <- beta[i]*xi + intp[i]
-                } else {
-                    xi <- (intp[j] - intp[i])/(beta[i] - beta[j])
-                    yi <- beta[i]*xi + intp[i]
-                }
-                ## rounding again
-                xi <- round(xi, 1e-6)
-                yi <- round(yi, 1e-6)
-
-                if (x0[i] < x1[i]) {
-                    if (xi <= x0[i] || xi >= x1[i]) next
-                } else {
-                    if (xi >= x0[i] || xi <= x1[i]) next
-                }
-
-                if (y0[i] < y1[i]) {
-                    if (yi <= y0[i] || yi >= y1[i]) next
-                } else {
-                    if (yi >= y0[i] || yi <= y1[i]) next
-                }
-
-                ## if we reach here, the intersection point is within
-                ## the 1st segment, check if it is within the 2nd one
-                if (x0[j] < x1[j]) {
-                    if (xi <= x0[j] || xi >= x1[j]) next
-                } else {
-                    if (xi >= x0[j] || xi <= x1[j]) next
-                }
-
-                if (y0[i] < y1[j]) {
-                    if (yi <= y0[j] || yi >= y1[j]) next
-                } else {
-                    if (yi >= y0[j] || yi <= y1[j]) next
-                }
-#                cat("i =", i, "j =", j, "\n")
-#                browser()
-                return(Inf)
+    if (!fast) {
+        fCollect <- function(i) {
+            ## find all nodes to move simultaneously
+            ii <- H[[i]]
+            if (!is.null(ii)) {
+                j <<- c(j, ii)
+                for (jj in ii) fCollect(jj)
             }
         }
-        Dang <- NULL
-        for (i in NODES) {
-            #cat("i =", i, "\n")
-            j <- c(link[l1 == i, 2], link[l2 == i, 1])
-            if (length(j) == 1) next
-            alpha <- atan2(yy[j] - yy[i], xx[j] - xx[i])
-            Dang <- c(Dang, diff(alpha))
-            #print(Dang)
+
+        ## NOTE: moving this function outside of the body of plot.haploNet() is not more efficient
+        energy <- function(xx, yy) {
+            ## First, check line crossings
+            nlink <- length(l1)
+            ## rounding makes it work better (why???)
+            x0 <- round(xx[l1])
+            y0 <- round(yy[l1])
+            x1 <- round(xx[l2])
+            y1 <- round(yy[l2])
+            ## compute all the slopes and intercepts:
+            beta <- (y1 - y0)/(x1 - x0)
+            if (any(is.na(beta))) return(Inf)
+            intp <- y0 - beta*x0
+            for (i in 1:(nlink - 1)) {
+                for (j in (i + 1):nlink) {
+                    ## in case they are parallel:
+                    if (beta[i] == beta[j]) next
+                    ## if both lines are vertical:
+                    if (abs(beta[i]) == Inf && abs(beta[j]) == Inf) next
+                    ## now find the point where both lines cross
+                    if (abs(beta[i]) == Inf) { # in case the 1st line is vertical...
+                        xi <- x0[i]
+                        yi <- beta[j]*xi + intp[j]
+                    } else if (abs(beta[j]) == Inf) { # ... or the 2nd one
+                        xi <- x0[j]
+                        yi <- beta[i]*xi + intp[i]
+                    } else {
+                        xi <- (intp[j] - intp[i])/(beta[i] - beta[j])
+                        yi <- beta[i]*xi + intp[i]
+                    }
+                    xi <- round(xi) # rounding as above
+                    yi <- round(yi)
+
+                    if (x0[i] < x1[i]) {
+                        if (xi <= x0[i] || xi >= x1[i]) next
+                    } else {
+                        if (xi >= x0[i] || xi <= x1[i]) next
+                    }
+
+                    if (y0[i] < y1[i]) {
+                        if (yi <= y0[i] || yi >= y1[i]) next
+                    } else {
+                        if (yi >= y0[i] || yi <= y1[i]) next
+                    }
+
+                    ## if we reach here, the intersection point is on
+                    ## the 1st segment, check if it is on the 2nd one
+                    if (x0[j] < x1[j]) {
+                        if (xi <= x0[j] || xi >= x1[j]) next
+                    } else {
+                        if (xi >= x0[j] || xi <= x1[j]) next
+                    }
+
+                    if (y0[i] < y1[j]) {
+                        if (yi <= y0[j] || yi >= y1[j]) next
+                    } else {
+                        if (yi >= y0[j] || yi <= y1[j]) next
+                    }
+                    return(Inf)
+                }
+            }
+            D <- dist(cbind(xx, yy))
+            sum(1/c(D)^2, na.rm = TRUE)
         }
-        #x <- (xx[l1] + xx[l2])/2 # add the edge
-        #y <- (yy[l1] + yy[l2])/2 # midpoints
-        D <- dist(cbind(xx, yy))
-        sum(1/c(D)^2, na.rm = TRUE)
-    }
 
-    Rotation <- function(rot, i, beta) {
-        ## rot: indices of the nodes to rotate
-        ## i: index of the node connected to 'rot' (= fixed rotation point)
-        xx.rot <- xx[rot] - xx[i]
-        yy.rot <- yy[rot] - yy[i]
-        theta <- atan2(yy.rot, xx.rot) + beta
-##        cat("i =", i, "rot =", rot, "theta =", theta, "beta =", beta, "\n")
-        h <- sqrt(xx.rot^2 + yy.rot^2)
-        new.xx[rot] <<- h*cos(theta) + xx[i]
-        new.yy[rot] <<- h*sin(theta) + yy[i]
-    }
+        Rotation <- function(rot, i, beta) {
+            ## rot: indices of the nodes to rotate
+            ## i: index of the node connected to 'rot' (= fixed rotation point)
+            xx.rot <- xx[rot] - xx[i]
+            yy.rot <- yy[rot] - yy[i]
+            theta <- atan2(yy.rot, xx.rot) + beta
+            h <- sqrt(xx.rot^2 + yy.rot^2)
+            new.xx[rot] <<- h*cos(theta) + xx[i]
+            new.yy[rot] <<- h*sin(theta) + yy[i]
+        }
 
-    OptimizeRotation <- function(node, rot) {
-        ## test the direction 1st
-        inc <- pi/90
-        Rotation(rot, node, inc)
-        new.E <- energy(new.xx, new.yy)
-        ##cat("node =", node, "rot = ", rot, "E =", E, "new.E =", new.E, "\n")
-        if (new.E >= E) {
-            inc <- -inc
+        OptimizeRotation <- function(node, rot) {
+            ## test the direction 1st
+            inc <- pi/90
             Rotation(rot, node, inc)
             new.E <- energy(new.xx, new.yy)
+            if (new.E >= E) {
+                inc <- -inc
+                Rotation(rot, node, inc)
+                new.E <- energy(new.xx, new.yy)
+            }
+
+            while (new.E < E) {
+                xx <<- new.xx
+                yy <<- new.yy
+                E <<- new.E
+                Rotation(rot, node, inc)
+                new.E <- energy(new.xx, new.yy)
+            }
         }
-        ##cat("node =", node, "rot = ", rot, "E =", E, "new.E =", new.E, "\n")
-        ##plot(xx, yy, type = "n")
-        ##segments(xx[l1], yy[l1], xx[l2], yy[l2], lwd = 3)
-        ##text(xx, yy, as.character(as.roman(1:n)), font = 2)
-        ##plot(new.xx, new.yy, type = "n")
-        ##segments(new.xx[l1], new.yy[l1], new.xx[l2], new.yy[l2], lwd = 3)
-        ##text(new.xx, new.yy, as.character(as.roman(1:n)), font = 2, col = "blue")
 
-        while (new.E < E) {
-            ##plot(xx, yy, type = "n")
-            ##segments(xx[l1], yy[l1], xx[l2], yy[l2], lwd = 3)
-            ##text(xx, yy, as.character(as.roman(1:n)), font = 2)
-            ##plot(new.xx, new.yy, type = "n")
-            ##segments(new.xx[l1], new.yy[l1], new.xx[l2], new.yy[l2], lwd = 3)
-            ##cat("node =", node, "E =", E, "new.E =", new.E, "\n")
-            xx <<- new.xx
-            yy <<- new.yy
-            E <<- new.E
-            Rotation(rot, node, inc)
-            new.E <- energy(new.xx, new.yy)
-        }
-    }
+        E <- energy(xx, yy)
+        new.xx <- xx
+        new.yy <- yy
 
-    NODES <- 1:n
-    E <- energy(xx, yy)
-    new.xx <- xx
-    new.yy <- yy
-###HH <<- H
-    nextOnes <- NULL
-    for (i in H[[central]][-1]) {
-        ## collect the nodes descending from 'i':
-        j <- NULL # j must be initialized before calling fCollect
-        fCollect(i)
-        rot <- c(i, j) # index of the nodes that will be moved
-        OptimizeRotation(central, rot)
-        nextOnes <- c(nextOnes, i)
-    }
-
-    while (!is.null(nextOnes)) {
-        newNodes <- nextOnes
         nextOnes <- NULL
-        for (i in newNodes) {
-            if (is.null(H[[i]])) next
-            for (j in H[[i]]) {
-                fCollect(j)
-                rot <- j
-                OptimizeRotation(i, rot)
-                nextOnes <- c(nextOnes, rot)
+        for (i in H[[central]][-1]) {
+            ## collect the nodes descending from 'i':
+            j <- NULL # j must be initialized before calling fCollect
+            fCollect(i)
+            rot <- c(i, j) # index of the nodes that will be moved
+            OptimizeRotation(central, rot)
+            nextOnes <- c(nextOnes, i)
+        }
+
+        while (!is.null(nextOnes)) {
+            newNodes <- nextOnes
+            nextOnes <- NULL
+            for (i in newNodes) {
+                if (is.null(H[[i]])) next
+                for (j in H[[i]]) {
+                    fCollect(j)
+                    rot <- j
+                    OptimizeRotation(i, rot)
+                    nextOnes <- c(nextOnes, rot)
+                }
             }
         }
     }
-}
 
     plot(xx, yy, type = "n", xlab = "", ylab = "",
          axes = FALSE, bty = "n", asp = asp, ...)
     segments(xx[l1], yy[l1], xx[l2], yy[l2], lwd = lwd,
              lty = lty, col = col.link)
 
-    ## from Klaus:
-    if (show.mutation) {
-        ld1 <- x[, 3]
-        ld2 <- x[, 3] * scale.ratio
-        for (i in seq_along(ld1)) {
-            pc <- ((1:ld1[i]) / (ld1[i] + 1) * ld2[i] + size[l1[i]]/2) / (ld2[i] + (size[l1[i]] + size[l2[i]])/2)
-            xr <- pc * (xx[l2[i]] - xx[l1[i]]) +  xx[l1[i]]
-            yr <- pc * (yy[l2[i]] - yy[l1[i]]) +  yy[l1[i]]
-            symbols(xr, yr, circles = rep(lwd/15, length(xr)), inches = FALSE, add = TRUE,
-                    fg = col.link, bg = col.link)
-        }
-    }
+    ## draw alternative links
+    altlink <- attr(x, "alter.links")
+    if (!is.null(altlink) && !identical(threshold, 0))
+        .drawAlternativeLinks(xx, yy, altlink, threshold, show.mutation)
 
-    if (is.null(pie))
-        symbols(xx, yy, circles = size/2, inches = FALSE,
-                add = TRUE, fg = col, bg = bg)
-    else {
-        nc <- ncol(pie)
-        co <- if (length(bg) == 1 && bg == "white") rainbow(nc) else rep(bg, length.out = nc)
-        for (i in 1:n)
-            floating.pie.asp(xx[i], yy[i], pie[i, ], radius = size[i]/2, col = co)
+    if (show.mutation)
+        .labelSegmentsHaploNet(xx, yy, link, x[, 3], size, lwd, col.link, as.numeric(show.mutation))
+
+    .drawSymbolsHaploNet(xx, yy, size, col, bg, pie)
+
+    if (labels) {
+        labels <- attr(x, "labels") # for export below
+        text(xx, yy, labels, font = font, cex = cex)
     }
-    if (labels)
-        text(xx, yy, attr(x, "labels"), font = font, cex = cex)
     if (legend[1]) {
         if (is.logical(legend)) {
             cat("Click where you want to draw the legend\n")
@@ -435,6 +515,8 @@ if (!fast) {
         }
         if (!is.null(pie)) {
             TEXT <- paste(" ", colnames(pie))
+            nc <- ncol(pie)
+            co <- if (length(bg) == 1 && bg == "white") rainbow(nc) else rep(bg, length.out = nc)
             for (i in 1:nc) {
                 Y <- xy[2] - 2 * (i + 1) * vspace
                 symbols(xy[1] + 0.5, Y, circles = 0.5,
@@ -443,6 +525,13 @@ if (!fast) {
             }
         }
     }
+    assign("last_plot.haploNet",
+           list(xx = xx, yy = yy, link = link, step = x[, 3], size = size,
+                col = col, bg = bg, lwd = lwd, lty = lty, col.link = col.link,
+                labels = labels, font = font, cex = cex, asp = asp, pie = pie,
+                show.mutation = show.mutation, alter.links = altlink,
+                threshold = threshold),
+           envir = .PlotHaploNetEnv)
 }
 
 plot.haplotype <- function(x, ...)
@@ -723,3 +812,4 @@ LD2 <- function(x, locus = 1:2, details = TRUE)
     }
     res
 }
+
